# ARPIST

**ARPIST** is an acronym for **A**utonomous **R**ocket **P**ointer **I**nteractive **S**imulation of **T**elemetry (**ARPIST** in short).

In brief, ARPIST is a tool for simulating the transmission of telemetry data from a local device connected via serial port to a board with a radio module equipped, in order to test ARP response to data in a testing environment.


## Getting started

### Prerequisites

- **Rust**: Rust need to be installed on the machine where you want to build and run ARPIST. You can find the installation instructions [here](https://www.rust-lang.org/tools/install).
- **C++**: The boardcore library, which is used by ARPIST, is written in C++. Consult the [Boardcore repo](https://git.skywarder.eu/avn/swd/skyward-boardcore) for the installation instructions.

### Installation

#### Host-side

If you want to install just ARPIST (host tool) on your machine, you can do it with the following command:

```bash
cargo install --git ssh://git@git.skywarder.eu/federico.lolli/arpist.git
```

If you plan instead to install also the board-side tool, you can clone first the repository:

```bash
git clone --recurse-submodules ssh://git@git.skywarder.eu/federico.lolli/arpist.git
```

And then install it with the following commands

```bash
cargo install --path ./on-host
```

#### Board-side

If you want to install ARPIST (board tool) on your board, you can do it with the following command:

```bash
git clone --recurse-submodules ssh://git@git.skywarder.eu/federico.lolli/arpist.git
```

Then you can move to the `on-device` directory and build it with the following command:

```bash
cd arpist/on-device
./sbs -f arpist-device-sender
```

The above procedure should flash an entrypoint capable of decoding any mavlink messages coming from serial and sending it to the radio.

### Developing

Both the host-side and the board-side scripts can be found in the justfile file for the sake of simplicity and ease of development.

First clone the repository:

```bash
git clone --recurse-submodules git@git.skywarder.eu:federico.lolli/arpist.git
cd arpist
```

#### Host-side

If you want to develop the host side of ARPIST, you can move to `on-host` directory and build it with the following command:

```bash
cargo build --release
```

The executable will be located in the `target/release` relative subfolder.

#### Board-side

If you want to develop the board side of ARPIST, you can move to `on-device` directory and build it with the following command:

```bash
./sbs -f arpist-device-sender
```

To flash the sender, if you want to test on a receiver board, you can flash a dummy receiver with the following command:

```bash
./sbs -f arpist-device-receiver
```

## Usage

ARPIST needs a board with a radio module connected to a serial port to work. The board must have a specific entrypoint flashed _(please refer to the installation paragraph above for this)_.

Once the board has the correct entrypoint flashed and is connected to the serial port, you can run ARPIST (host-side) as follows, after installing it.

The most common use case is to parse two CSV file containing telemetry data and send it to the board, one low rate (`ROCKET_STATS_TM`) and one high rate (`ROCKET_FLIGHT_TM`). You can do it with the following command:

```bash
arpist <low_rate_telemetry_file> <high_rate_telemetry_file>
```

All available logs (correctly formatted) are found in the `logs` folder.

**NOTE**: by default ARPIST will wait for a synchronization signal from the board before sending further data. If you want to disable this behavior, you can use the `--no-sync` option (or just the short `-n` flag).

```bash
arpist -n <low_rate_telemetry_file> <high_rate_telemetry_file>
```

By adding the `--verbose` option (or just the short `-v` flag), you can enable the verbose mode, which will print the sent packets to the standard output.

```bash
arpist -v <low_rate_telemetry_file> <high_rate_telemetry_file>
```

**NOTE**: if a single STM32 compatible board is connected to the serial port, ARPIST will automatically detect it and use it, so you can omit the `--port <serial_port>` option.

### Detailed Usage

```
Usage: arpist [OPTIONS] <LOW_RATE_FILE> <HIGH_RATE_FILE>

Arguments:
  <LOW_RATE_FILE>   The low rate telemetry CSV file to read from
  <HIGH_RATE_FILE>  The high rate telemetry CSV file to read from

Options:
  -p, --port <PORT>            The serial port to use
  -b, --baud-rate <BAUD_RATE>  baud rate [default: 115200]
  -n, --no-sync                disable SYNC MODE (wait for a sync signal from the device before proceding)
  -v, --verbose                The log level
  -h, --help                   Print help
```
